/* This is an alternative API to MPGLIB that was implemented by Sonos */

// 1/20/2007 (RK) - Repackaged all of this code, which was designed as standard
// C with global variables for internal state, as a C++ class.  This allows us
// to decode to MP3s at the same time so that we can perform a crossfade.

#ifndef MPGLIB_H
#define MPGLIB_H

#include "mpg123.h"

struct III_sideinfo;
struct gr_info_s;

class MPGLib : public frame_callback
{
public:
    MPGLib();
    virtual ~MPGLib() {}

    static void init();

    unsigned int process_frame(const unsigned char* frame_data,
                               unsigned int frame_len,
                               short* pcm_data);

private:
    void set_pointer(int ssize, long backstep);

    struct mpstr mp;
    struct frame fr;

    int bsbufend[2];
    int bsbufold_end;
    unsigned char bsspace[2][MAXFRAMESIZE + 512];
    unsigned char *bsbuf, *bsbufold;
    int bsnum;

    struct bitstream_info bsi;

    int pcm_point;
    unsigned char *pcm_sample;

    int frame_num;
    int used_reservoir;

    // DECODE.C:

    void synth_reset();

    // frame_callback overrides
    virtual int synth(real *bandPtr,int channel,unsigned char *out,int *pnt);
    virtual int synth_mono(real *bandPtr,unsigned char *samples,int *pnt);

    int bo;
    real buffs[2][2][0x110];

    // LAYER3.C:

    int III_get_side_info(struct III_sideinfo *,int, int, long, int, int);
    int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_info);
    int III_get_scale_factors_2(int *scf,struct gr_info_s *gr_info,int i_stereo);
    int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
                              struct gr_info_s *gr_info,int sfreq,int part2bits);
    void III_i_stereo(real xr_buf[2][SBLIMIT][SSLIMIT],int *scalefac,
                      struct gr_info_s *gr_info,int sfreq,int ms_stereo,int lsf);
    void III_antialias(real xr[SBLIMIT][SSLIMIT],struct gr_info_s *gr_info);
    void III_hybrid(struct mpstr *mp, real fsIn[SBLIMIT][SSLIMIT],
                    real tsOut[SSLIMIT][SBLIMIT],
                    int ch,struct gr_info_s *gr_info,struct frame *fr);
    int do_layer3(struct mpstr *mp,struct frame *fr);

private:
    // Do not copy
    MPGLib(const MPGLib&);
    MPGLib& operator=(const MPGLib&);
};

#endif
